@@ -4,7 +4,8 @@ import sqlite3
 import os
 import sys
 import collections
-
+import re
+from unidecode import unidecode
 from bx.bbi.bigwig_file import BigWigFile
 from gemini.config import read_gemini_config
 
@@ -44,11 +45,13 @@ def get_anno_files():
                                            'encode.6celltypes.consensus.bedg.gz'),
      'gerp_elements': os.path.join(anno_dirname, 'hg19.gerp.elements.bed.gz'),
      'vista_enhancers': os.path.join(anno_dirname, 'hg19.vista.enhancers.20131108.bed.gz'),
-     'cosmic': os.path.join(anno_dirname, 'hg19.cosmic.v67.20131024.gz'),     
+     'cosmic': os.path.join(anno_dirname, 'hg19.cosmic.v67.20131024.gz')
     }
     # optional annotations
     if os.path.exists(os.path.join(anno_dirname, 'hg19.gerp.bw')):
         annos['gerp_bp'] = os.path.join(anno_dirname, 'hg19.gerp.bw')
+    if os.path.exists(os.path.join(anno_dirname, 'whole_genome_SNVs.tsv.compressed.gz')):
+        annos['cadd_score'] = os.path.join(anno_dirname, 'whole_genome_SNVs.tsv.compressed.gz')
     return annos
 
 class ClinVarInfo(object):
@@ -150,7 +153,6 @@ ThousandGInfo = collections.namedtuple("ThousandGInfo",
                                         aaf_AFR \
                                         aaf_EUR")
 
-
 def load_annos():
     """
     Populate a dictionary of Tabixfile handles for
@@ -254,6 +256,18 @@ def _get_var_coords(var, naming):
         chrom = _get_chr_as_grch37(chrom)
     return chrom, start, end
 
+def _get_cadd_scores(var, labels, hit):
+    """
+    get cadd scores
+    """
+    raw = hit[3].split(",")
+    scaled = hit[4].split(",")
+    
+    p = re.compile(str(var.ALT[0]))
+    for m in p.finditer(str(labels[hit[2]])):
+        pos = m.start()
+        return raw[pos], scaled[pos]
+        
 
 def annotations_in_region(var, anno, parser_type=None, naming="ucsc"):
     """Iterator of annotations found in a genomic region.
@@ -277,7 +291,6 @@ def bigwig_summary(var, anno, naming="ucsc"):
     return _get_bw_summary(coords, anno)
 
 
-
 # ## Track-specific annotations
 def get_cpg_island_info(var):
     """
@@ -366,6 +379,36 @@ def get_vista_enhancers(var):
     return ",".join(vista_enhancers) if len(vista_enhancers) > 0 else None
 
 
+def get_cadd_scores(var):
+    """
+    Returns the C-raw scores & scaled scores (CADD) to predict deleterious
+    variants. Implemented only for SNV's
+    """
+    if "cadd_score" not in annos:
+        raise IOError("Need to download the CADD data file for deleteriousness."
+                      "Run `gemini update --dataonly --extra cadd_score")
+
+    cadd_raw = cadd_scaled = None
+    labels = {"A":"CGT", "C":"AGT", "G":"ACT", "T":"ACG", "R":"ACGT", "M":"ACGT"}
+    
+    for hit in annotations_in_region(var, "cadd_score", "tuple", "grch37"):
+        # we want exact position mapping here and not a range (end-start) as
+        # returned in hit (e.g. indels) & we do not want to consider del & ins
+        if str(hit[1]) == str(var.POS) and len(var.REF) == 1 and \
+           len(var.ALT[0]) == 1:
+           
+            if str(hit[2]) == var.REF and str(var.ALT[0]) in labels[hit[2]]:
+               (cadd_raw, cadd_scaled) = _get_cadd_scores(var, labels, hit)
+            
+            # consider ref cases with ambiguity codes R (G,A) and M (A,C)
+            elif ((str(hit[2]) == 'R'  and var.REF in('G','A')) or \
+                (str(hit[2]) == 'M'  and var.REF in('A','C'))) and \
+                str(var.ALT[0]) in labels[hit[2]]:
+                (cadd_raw, cadd_scaled) = _get_cadd_scores(var, labels, hit)
+    
+    return (cadd_raw, cadd_scaled)
+     
+    
 def get_pfamA_domains(var):
     """
     Returns pfamA domains that a variant overlaps
@@ -431,7 +474,9 @@ def get_clinvar_info(var):
             else:
                 info_map[info] = True
 
-        clinvar.clinvar_dbsource = info_map['CLNSRC'] or None
+        raw_dbsource = info_map['CLNSRC'] or None
+        #interpret 8-bit strings and convert to plain text
+        clinvar.clinvar_dbsource = unidecode(raw_dbsource.decode('utf-8'))
         clinvar.clinvar_dbsource_id = info_map['CLNSRCID'] or None
         clinvar.clinvar_origin           = \
             clinvar.lookup_clinvar_origin(info_map['CLNORIGIN'])
@@ -439,12 +484,10 @@ def get_clinvar_info(var):
             clinvar.lookup_clinvar_significance(info_map['CLNSIG'])
         clinvar.clinvar_dsdb = info_map['CLNDSDB'] or None
         clinvar.clinvar_dsdbid = info_map['CLNDSDBID'] or None
-        # Clinvar represents commas as \x2c.  Make them commas.
         # Remap all unicode characters into plain text string replacements
         raw_disease_name = info_map['CLNDBN'] or None
-        #raw_disease_name.decode('string_escape')
-        clinvar.clinvar_disease_name = \
-            unicode(raw_disease_name, errors="replace").encode(errors="replace")
+        clinvar.clinvar_disease_name = unidecode(raw_disease_name.decode('utf-8'))
+        # Clinvar represents commas as \x2c.  Make them commas.
         clinvar.clinvar_disease_name = clinvar.clinvar_disease_name.decode('string_escape')
 
         clinvar.clinvar_disease_acc = info_map['CLNACC'] or None
@@ -475,6 +518,13 @@ def get_dbsnp_info(var):
 def get_esp_info(var):
     """
     Returns a suite of annotations from the ESP project
+
+    ESP reports the minor allele frequency (MAF), not the
+    alternate allele frequency (AAF). We must therefore figure
+    out whther the reference or alternate allele is the minor allele.
+
+    1       69496   rs150690004     G       A       .       PASS    DBSNP=dbSNP_134;EA_AC=2,6764;AA_AC=23,3785;TAC=25,10549;MAF=0.0296,0.604,0.2364;GTS=AA,AG,GG;EA_GTC=0,2,3381;AA_GTC=5,13,1886;GTC=5,15,5267;DP=91;GL=OR4F5;CP=0.5;CG=2.3;AA=G;CA=.;EXOME_CHIP=no;GWAS_PUBMED=.;GM=NM_001005484.1;FG=missense;AAC=SER/GLY;PP=136/306;CDP=406;GS=56;PH=benign
+    1       69511   rs75062661      A       G       .       PASS    DBSNP=dbSNP_131;EA_AC=5337,677;AA_AC=1937,1623;TAC=7274,2300;MAF=11.2571,45.5899,24.0234;GTS=GG,GA,AA;EA_GTC=2430,477,100;AA_GTC=784,369,627;GTC=3214,846,727;DP=69;GL=OR4F5;CP=1.0;CG=1.1;AA=G;CA=.;EXOME_CHIP=no;GWAS_PUBMED=.;GM=NM_001005484.1;FG=missense;AAC=ALA/THR;PP=141/306;CDP=421;GS=58;PH=benign
     """
     aaf_EA = aaf_AA = aaf_ALL = None
     maf = fetched = con = []
@@ -496,14 +546,20 @@ def get_esp_info(var):
                     # SA=http://www.ncbi.nlm.nih.gov/sites/varvu?gene=4524&amp%3Brs=1801131|http://omim.org/entry/607093#0004
                         (key, value) = info.split("=", 1)
                         info_map[key] = value
-                # get the % minor allele frequencies
-                if info_map.get('MAF') is not None:
-                    lines = info_map['MAF'].split(",")
-                    # divide by 100 because ESP reports allele
-                    # frequencies as percentages.
-                    aaf_EA = float(lines[0]) / 100.0
-                    aaf_AA = float(lines[1]) / 100.0
-                    aaf_ALL = float(lines[2]) / 100.0
+
+                # get the allele counts so that we can compute alternate allele frequencies
+                # example: EA_AC=2,6764;AA_AC=23,3785;TAC=25,10549
+                if info_map.get('EA_AC') is not None:
+                    lines = info_map['EA_AC'].split(",")
+                    aaf_EA = float(lines[0]) / (float(lines[0]) + float(lines[1]))
+
+                if info_map.get('AA_AC') is not None:
+                    lines = info_map['AA_AC'].split(",")
+                    aaf_AA = float(lines[0]) / (float(lines[0]) + float(lines[1]))
+
+                if info_map.get('TAC') is not None:
+                    lines = info_map['TAC'].split(",")
+                    aaf_ALL = float(lines[0]) / (float(lines[0]) + float(lines[1]))
 
                 # Is the SNP on an human exome chip?
                 if info_map.get('EXOME_CHIP') is not None and \
@@ -512,6 +568,7 @@ def get_esp_info(var):
                 elif info_map.get('EXOME_CHIP') is not None and \
                         info_map['EXOME_CHIP'] == "yes":
                     exome_chip = 1
+
     return ESPInfo(found, aaf_EA, aaf_AA, aaf_ALL, exome_chip)
 
 
