@@ -3,7 +3,10 @@
 import os
 import sys
 import sqlite3
-from collections import defaultdict
+from collections import defaultdict, namedtuple
+import json
+import subprocess
+
 import numpy as np
 from scipy.stats import mode
 import pysam
@@ -85,10 +88,8 @@ def _annotate_variants(args, conn, get_val_fn, col_names=None, col_types=None, c
 
             # update_data starts out as a list of the values that should
             # be used to populate the new columns for the current row.
-            update_data = get_val_fn(annotations_in_region(row,
-                                                    anno,
-                                                    "tuple",
-                                                    naming))
+            update_data = get_val_fn(annotations_in_region(row, anno,
+                                                           "tuple", naming))
             # were there any hits for this row?
             if len(update_data) > 0:
                 # we add the primary key to update_data for the
@@ -207,24 +208,25 @@ def annotate_variants_extract(args, conn, col_names, col_types, col_ops, col_idx
                 try:
                     vals.append(int(val))
                 except ValueError:
-                    sys.exit ('Non-integer value found in annotation file: %s\n' % (val))
+                    if not val:
+                        vals.append(None)
+                    else:
+                        sys.exit('Non-integer value found in annotation file: %s\n' % (val))
             elif col_types[idx] == "float":
                 try:
                     vals.append(float(val))
                 except ValueError:
-                    sys.exit ('Non-float value found in annotation file: %s\n' % (val))
+                    if not val:
+                        vals.append(None)
+                    else:
+                        sys.exit('Non-float value found in annotation file: %s\n' % (val))
             else:
                 vals.append(val)
 
         return vals
-
-
     return _annotate_variants(args, conn, summarize_hits,
                               col_names, col_types, col_ops)
 
-
-
-
 def annotate(parser, args):
 
     def _validate_args(args):
@@ -242,31 +244,29 @@ def annotate(parser, args):
         col_ops = args.col_operations.split(',')
         col_names = args.col_names.split(',')
         col_types = args.col_types.split(',')
-        col_idxs  = args.col_extracts.split(',')
+        col_idxs = args.col_extracts.split(',')
 
         supported_types = ['text', 'float', 'integer']
         for col_type in col_types:
             if col_type not in supported_types:
-                sys.exit('EXITING: Column type [%s] not supported.\n' % \
+                sys.exit('EXITING: Column type [%s] not supported.\n' %
                          (col_type))
 
-        supported_ops = ['mean', 'median', 'mode', 'min', 'max', 'first', \
+        supported_ops = ['mean', 'median', 'mode', 'min', 'max', 'first',
                          'last', 'list', 'uniq_list']
         for col_op in col_ops:
             if col_op not in supported_ops:
-                sys.exit('EXITING: Column operation [%s] not supported.\n' % \
+                sys.exit('EXITING: Column operation [%s] not supported.\n' %
                          (col_op))
 
-        if not (len(col_ops) == len(col_names) == \
+        if not (len(col_ops) == len(col_names) ==
                 len(col_types) == len(col_idxs)):
             sys.exit('EXITING: The number of column names, numbers, types, and '
-                     'operations must match: [%s], [%s], [%s], [%s]\n' % \
+                     'operations must match: [%s], [%s], [%s], [%s]\n' %
                      (args.col_names, args.col_extracts, args.col_types, args.col_operations))
 
         return col_names, col_types, col_ops, col_idxs
 
-
-
     if (args.db is None):
         parser.print_help()
         exit(1)
@@ -303,3 +303,83 @@ def annotate(parser, args):
     for col_name in col_names:
         with database_transaction(args.db) as c:
             c.execute('''create index %s on variants(%s)''' % (col_name + "idx", col_name))
+
+# ## Automate addition of extra fields to database
+
+def add_extras(gemini_db, chunk_dbs):
+    """Annotate gemini database with extra columns from processed chunks, if available.
+    """
+    extra_files = []
+    header_files = []
+    for chunk in chunk_dbs:
+        extra_file, header_file = get_extra_files(chunk)
+        if os.path.exists(extra_file):
+            extra_files.append(extra_file)
+            assert os.path.exists(header_file)
+            header_files.append(header_file)
+    if header_files:
+        header, types = _merge_headers(header_files)
+        ops = ["first" for t in types]
+        extra_beds = [_json_to_bed(x, header) for x in extra_files]
+        final_bed = _merge_beds(extra_beds, gemini_db)
+        Args = namedtuple("Args", "db,anno_file,anno_type,col_operations,col_names,col_types,col_extracts")
+        args = Args(gemini_db, final_bed, "extract", ",".join(ops),
+                    ",".join(header), ",".join(types),
+                    ",".join([str(i + 4) for i in range(len(header))]))
+        annotate(None, args)
+        for fname in extra_beds + [final_bed, final_bed + ".tbi"] + header_files + extra_files:
+            if os.path.exists(fname):
+                os.remove(fname)
+
+def _merge_beds(in_beds, final_db):
+    """Merge BED files into a final sorted output file.
+    """
+    if len(in_beds) == 1:
+        out_file = in_beds[0]
+    else:
+        out_file = "%s.bed" % os.path.splitext(final_db)[0]
+        cmd = "cat %s | sort -k1,1 -k2,2n > %s" % (" ".join(in_beds), out_file)
+        subprocess.check_call(cmd, shell=True)
+    subprocess.check_call(["bgzip", "-f", out_file])
+    bgzip_out = out_file + ".gz"
+    subprocess.check_call(["tabix", "-p", "bed", "-f", bgzip_out])
+    return bgzip_out
+
+def _json_to_bed(fname, header):
+    """Convert JSON output into a BED file in preparation for annotation.
+    """
+    out_file = "%s.bed" % os.path.splitext(fname)[0]
+    with open(fname) as in_handle:
+        with open(out_file, "w") as out_handle:
+            for line in in_handle:
+                cur_info = json.loads(line)
+                parts = [str(cur_info.get(h, "")) for h in ["chrom", "start", "end"] + header]
+                out_handle.write("\t".join(parts) + "\n")
+    return out_file
+
+def _merge_headers(header_files):
+    """Merge a set of header files into a single final header for annotating.
+    """
+    ignore = set(["chrom", "start", "end"])
+    ctype_order = ["text", "float", "integer", None]
+    out = {}
+    for h in header_files:
+        with open(h) as in_handle:
+            header = json.loads(in_handle.read())
+            for column, ctype in header.items():
+                if column not in ignore:
+                    cur_ctype = sorted([ctype, out.get(column)], key=lambda x: ctype_order.index(x))[0]
+                    out[column] = cur_ctype
+    headers = []
+    types = []
+    for header in sorted(out.keys()):
+        headers.append(header)
+        types.append(out[header])
+    return headers, types
+
+def get_extra_files(gemini_db):
+    """Retrieve extra file names associated with a gemini database, for flexible loading.
+    """
+    extra_file = "%s-extra.json" % os.path.splitext(gemini_db)[0]
+    extraheader_file = "%s-extraheader.json" % os.path.splitext(gemini_db)[0]
+    return extra_file, extraheader_file
