@@ -691,7 +691,7 @@ rm obs exp
 ###########################################################################
 # 39. Test the loading of gene_detailed table with multiple cores
 ###########################################################################
-echo "    query.t34...\c"
+echo "    query.t39...\c"
 echo "variant_id	chrom	gene	transcript_status	transcript	transcript_start	transcript_end	synonym	rvis_pct	protein_length	impact
 46	chr1	SAMD11	KNOWN	ENST00000342066	861118	879955	MGC45873	None	681	frame_shift
 578	chr1	TNFRSF18	PUTATIVE	ENST00000486728	1139224	1141060	AITR,CD357,GITR	None	169	frame_shift
@@ -710,5 +710,49 @@ check obs exp
 rm obs exp
 
 ############################################################################
- 
+# 40. Test phenotype column of the gene_table
+############################################################################
+echo "    query.t40...\c"
+echo "chrom	end	gene	mam_phenotype_id
+chr1	949422	ISG15	MP:0005390
+chr1	949608	ISG15	MP:0005390
+chr1	949655	ISG15	MP:0005390
+chr1	949832	ISG15	MP:0005390
+chr1	977356	AGRN	MP:0005378,MP:0005386,MP:0005388,MP:0005367,MP:0005369,MP:0005371,MP:0003631,MP:0002873,MP:0010768
+chr1	978762	AGRN	MP:0005378,MP:0005386,MP:0005388,MP:0005367,MP:0005369,MP:0005371,MP:0003631,MP:0002873,MP:0010768
+chr1	979594	AGRN	MP:0005378,MP:0005386,MP:0005388,MP:0005367,MP:0005369,MP:0005371,MP:0003631,MP:0002873,MP:0010768
+chr1	979748	AGRN	MP:0005378,MP:0005386,MP:0005388,MP:0005367,MP:0005369,MP:0005371,MP:0003631,MP:0002873,MP:0010768
+chr1	981131	AGRN	MP:0005378,MP:0005386,MP:0005388,MP:0005367,MP:0005369,MP:0005371,MP:0003631,MP:0002873,MP:0010768
+chr1	982205	AGRN	MP:0005378,MP:0005386,MP:0005388,MP:0005367,MP:0005369,MP:0005371,MP:0003631,MP:0002873,MP:0010768
+chr1	983005	AGRN	MP:0005378,MP:0005386,MP:0005388,MP:0005367,MP:0005369,MP:0005371,MP:0003631,MP:0002873,MP:0010768
+chr1	984711	AGRN	MP:0005378,MP:0005386,MP:0005388,MP:0005367,MP:0005369,MP:0005371,MP:0003631,MP:0002873,MP:0010768
+chr1	984957	AGRN	MP:0005378,MP:0005386,MP:0005388,MP:0005367,MP:0005369,MP:0005371,MP:0003631,MP:0002873,MP:0010768
+chr1	985378	AGRN	MP:0005378,MP:0005386,MP:0005388,MP:0005367,MP:0005369,MP:0005371,MP:0003631,MP:0002873,MP:0010768
+chr1	985826	AGRN	MP:0005378,MP:0005386,MP:0005388,MP:0005367,MP:0005369,MP:0005371,MP:0003631,MP:0002873,MP:0010768
+chr1	986885	AGRN	MP:0005378,MP:0005386,MP:0005388,MP:0005367,MP:0005369,MP:0005371,MP:0003631,MP:0002873,MP:0010768
+chr1	990213	AGRN	MP:0005378,MP:0005386,MP:0005388,MP:0005367,MP:0005369,MP:0005371,MP:0003631,MP:0002873,MP:0010768
+chr1	1139216	TNFRSF18	MP:0005397,MP:0005384,MP:0005387
+chr1	1139498	TNFRSF18	MP:0005397,MP:0005384,MP:0005387
+chr1	1140813	TNFRSF18	MP:0005397,MP:0005384,MP:0005387
+chr1	1147337	TNFRSF4	MP:0005384,MP:0005397,MP:0005378,MP:0002873,MP:0005388,MP:0005370,MP:0005387,MP:0005381
+chr1	1149480	TNFRSF4	MP:0005384,MP:0005397,MP:0005378,MP:0002873,MP:0005388,MP:0005370,MP:0005387,MP:0005381" > exp
+
+gemini query --header -q "select v.chrom, v.end, v.gene, g.mam_phenotype_id from variants v, \
+	                      gene_summary g where v.chrom=g.chrom and v.gene=g.gene and \
+						  v.impact_severity !='LOW' and mam_phenotype_id !='None'" test.query.db > obs
+
+check obs exp
+rm obs exp
+
+
+
+
+
+
+
+
+
+
+
+
  
\ No newline at end of file
